@@ -24,6 +24,13 @@
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 
+/**
+ * SECTION:goc-polygon
+ * @short_description: Polygon.
+ *
+ * #GocPolygon implements polygon drawing in the canvas.
+**/
+
 enum {
 	POLYGON_PROP_0,
 	POLYGON_PROP_POINTS,
@@ -219,7 +226,9 @@ goc_polygon_init_style (G_GNUC_UNUSED GocStyledItem *item, GOStyle *style)
 	if (style->fill.auto_type)
 		style->fill.type  = GO_STYLE_FILL_PATTERN;
 	if (style->fill.auto_fore)
-		go_pattern_set_solid (&style->fill.pattern, GO_COLOR_WHITE);
+		style->fill.pattern.fore = GO_COLOR_BLACK;
+	if (style->fill.auto_back)
+		style->fill.pattern.back = GO_COLOR_WHITE;
 }
 
 static void
@@ -240,12 +249,12 @@ goc_polygon_class_init (GocItemClass *item_klass)
                  g_param_spec_boxed ("points", _("points"), _("The polygon vertices"),
 				     GOC_TYPE_POINTS,
 				     GSF_PARAM_STATIC | G_PARAM_READWRITE));
-	g_object_class_install_property (obj_klass, POLYGON_PROP_SPLINE,
+/*	g_object_class_install_property (obj_klass, POLYGON_PROP_SPLINE,
 		g_param_spec_boolean ("use-spline",
 				      _("Use spline"),
 				      _("Use a Bezier closed cubic spline as contour"),
 				      FALSE,
-				      GSF_PARAM_STATIC | G_PARAM_READABLE));
+				      GSF_PARAM_STATIC | G_PARAM_READABLE));*/
 
 	item_klass->update_bounds = goc_polygon_update_bounds;
 	item_klass->distance = goc_polygon_distance;
