@@ -62,7 +62,7 @@ static const struct {
 	{ "pseudo-3d",	GOG_AXIS_SET_XY_pseudo_3d},
 	{ "xy-color",	GOG_AXIS_SET_XY_COLOR}
 };
-	
+
 GogAxisSet
 gog_axis_set_from_str (char const *str)
 {
@@ -73,13 +73,13 @@ gog_axis_set_from_str (char const *str)
 	if (str == NULL)
 		return GOG_AXIS_SET_NONE;
 
-	for (i = 0; i < G_N_ELEMENTS (axis_set_desc); i++) 
+	for (i = 0; i < G_N_ELEMENTS (axis_set_desc); i++)
 		if (strcmp (axis_set_desc[i].name, str) == 0) {
 			axis_set = axis_set_desc[i].axis_set;
 			found = TRUE;
 			break;
 		}
-	if (!found) 
+	if (!found)
 		g_warning ("[GogAxisSet::from_str] unknown axis set (%s)", str);
 	return axis_set;
 }
@@ -90,6 +90,10 @@ enum {
 	CHART_PROP_CARDINALITY_VALID,
 	CHART_PROP_PLOT_AREA,
 	CHART_PROP_PLOT_AREA_IS_MANUAL,
+	CHART_PROP_X_POS,
+	CHART_PROP_Y_POS,
+	CHART_PROP_ROWS,
+	CHART_PROP_COLUMNS
 };
 
 static GType gog_chart_view_get_type (void);
@@ -128,6 +132,7 @@ gog_chart_set_property (GObject *obj, guint param_id,
 	GogChart *chart = GOG_CHART (obj);
 	char **str_doubles;
 	char const *str;
+	gboolean changed = FALSE;
 
 	switch (param_id) {
 	case CHART_PROP_PLOT_AREA:
@@ -146,9 +151,30 @@ gog_chart_set_property (GObject *obj, guint param_id,
 	case CHART_PROP_PLOT_AREA_IS_MANUAL:
 		chart->is_plot_area_manual = g_value_get_boolean (value);
 		break;
+	case CHART_PROP_X_POS:
+		chart->x_pos = g_value_get_int (value);
+		changed = TRUE;
+		break;
+	case CHART_PROP_Y_POS:
+		chart->y_pos = g_value_get_int (value);
+		changed = TRUE;
+		break;
+	case CHART_PROP_COLUMNS:
+		chart->cols = g_value_get_int (value);
+		changed = TRUE;
+		break;
+	case CHART_PROP_ROWS:
+		chart->rows = g_value_get_int (value);
+		changed = TRUE;
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
 	}
+
+	if (changed) {
+		gog_graph_validate_chart_layout (GOG_GRAPH (GOG_OBJECT (chart)->parent));
+		gog_object_emit_changed (GOG_OBJECT (obj), TRUE);
+	}
 }
 
 static void
@@ -178,6 +204,18 @@ gog_chart_get_property (GObject *obj, guint param_id,
 	case CHART_PROP_PLOT_AREA_IS_MANUAL:
 		g_value_set_boolean (value, chart->is_plot_area_manual);
 		break;
+	case CHART_PROP_X_POS:
+		g_value_set_int (value, chart->x_pos);
+		break;
+	case CHART_PROP_Y_POS:
+		g_value_set_int (value, chart->y_pos);
+		break;
+	case CHART_PROP_COLUMNS:
+		g_value_set_int (value, chart->cols);
+		break;
+	case CHART_PROP_ROWS:
+		g_value_set_int (value, chart->rows);
+		break;
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -185,9 +223,9 @@ gog_chart_get_property (GObject *obj, guint param_id,
 }
 
 static void
-gog_chart_populate_editor (GogObject *gobj, 
-			   GOEditor *editor, 
-			   G_GNUC_UNUSED GogDataAllocator *dalloc, 
+gog_chart_populate_editor (GogObject *gobj,
+			   GOEditor *editor,
+			   G_GNUC_UNUSED GogDataAllocator *dalloc,
 			   GOCmdContext *cc)
 {
 	static guint chart_pref_page = 0;
@@ -212,6 +250,13 @@ gog_chart_children_reordered (GogObject *obj)
 	gog_chart_request_cardinality_update (chart);
 }
 
+static gboolean
+role_plot_can_add (GogObject const *parent)
+{
+	GogChart *chart = GOG_CHART (parent);
+	return (chart->axis_set & (1 << GOG_AXIS_Z)) == 0 || g_slist_length (chart->plots) == 0;
+}
+
 static void
 role_plot_post_add (GogObject *parent, GogObject *plot)
 {
@@ -244,7 +289,7 @@ role_plot_post_add (GogObject *parent, GogObject *plot)
 	/* APPEND to keep order, there won't be that many */
 	chart->plots = g_slist_append (chart->plots, plot);
 	gog_chart_request_cardinality_update (chart);
-	
+
 	if (chart->plots->next == NULL)
 		ok = gog_chart_axis_set_assign (chart,
 			gog_plot_axis_set_pref (GOG_PLOT (plot)));
@@ -262,13 +307,13 @@ role_plot_pre_remove (GogObject *parent, GogObject *plot)
 	gog_plot_axis_clear (GOG_PLOT (plot), GOG_AXIS_SET_ALL);
 	chart->plots = g_slist_remove (chart->plots, plot);
 	gog_chart_request_cardinality_update (chart);
-	
+
 	if (chart->plots == NULL)
 		gog_chart_axis_set_assign (chart, GOG_AXIS_SET_UNKNOWN);
 
-	if (chart->grid != NULL && 
+	if (chart->grid != NULL &&
 	    chart->axis_set != GOG_AXIS_SET_XY &&
-	    chart->axis_set != GOG_AXIS_SET_X && 
+	    chart->axis_set != GOG_AXIS_SET_X &&
 	    chart->axis_set != GOG_AXIS_SET_XY_pseudo_3d &&
 	    chart->axis_set != GOG_AXIS_SET_XY_COLOR &&
 	    chart->axis_set != GOG_AXIS_SET_RADAR) {
@@ -443,15 +488,15 @@ static GogObjectRole const roles[] = {
 	  { GOG_AXIS_COLOR } },
 	{ N_("Plot"), "GogPlot",	6,	/* keep the axis before the plots */
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_TYPE,
-	  NULL, NULL, NULL, role_plot_post_add, role_plot_pre_remove, NULL, { -1 } },
+	  role_plot_can_add, NULL, NULL, role_plot_post_add, role_plot_pre_remove, NULL, { -1 } },
 	{ N_("Title"), "GogLabel",	0,
-	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL, 
-	  GOG_POSITION_N|GOG_POSITION_ALIGN_CENTER, 
+	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL,
+	  GOG_POSITION_N|GOG_POSITION_ALIGN_CENTER,
 	  GOG_OBJECT_NAME_BY_ROLE,
 	  NULL, NULL, NULL, NULL, NULL, NULL, { -1 } },
 	{ N_("Legend"), "GogLegend",	11,
-	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL, 
-	  GOG_POSITION_E|GOG_POSITION_ALIGN_CENTER, 
+	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL,
+	  GOG_POSITION_E|GOG_POSITION_ALIGN_CENTER,
 	  GOG_OBJECT_NAME_BY_ROLE,
 	  NULL, NULL, NULL, NULL, NULL, NULL, { -1 } },
 #ifdef GOFFICE_WITH_LASEM
@@ -481,23 +526,40 @@ gog_chart_class_init (GogObjectClass *gog_klass)
 	gog_klass->can_manual_size = TRUE;
 
 	g_object_class_install_property (gobject_klass, CHART_PROP_CARDINALITY_VALID,
-		g_param_spec_boolean ("cardinality-valid", 
+		g_param_spec_boolean ("cardinality-valid",
 				      _("Valid cardinality"),
 				      _("Is the charts cardinality currently valid"),
-				      FALSE, 
+				      FALSE,
 				      GSF_PARAM_STATIC | G_PARAM_READABLE));
 	g_object_class_install_property (gobject_klass, CHART_PROP_PLOT_AREA,
-		g_param_spec_string ("plot-area", 
+		g_param_spec_string ("plot-area",
 				     _("Plot area"),
 				     _("Position and size of plot area, in percentage of chart size"),
-				     "0 0 1 1", 
+				     "0 0 1 1",
 				     GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, CHART_PROP_PLOT_AREA_IS_MANUAL,
-		g_param_spec_boolean ("is-plot-area-manual", 
-				      _("Manual plot area"), 
+		g_param_spec_boolean ("is-plot-area-manual",
+				      _("Manual plot area"),
 				      _("Is plot area manual"),
-				      FALSE, 
+				      FALSE,
 				      GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, CHART_PROP_X_POS,
+		g_param_spec_int ("xpos", _("xpos"),
+			_("Horizontal chart position in graph grid"),
+			0, G_MAXINT, 0, G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	/* we need to force saving of ypos since the default is not constant */
+	g_object_class_install_property (gobject_klass, CHART_PROP_Y_POS,
+		g_param_spec_int ("ypos", _("ypos"),
+			_("Vertical chart position in graph grid"),
+			0, G_MAXINT, 0, G_PARAM_READWRITE | GO_PARAM_PERSISTENT | GOG_PARAM_FORCE_SAVE));
+	g_object_class_install_property (gobject_klass, CHART_PROP_COLUMNS,
+		g_param_spec_int ("columns", _("columns"),
+			_("Number of columns in graph grid"),
+			1, G_MAXINT, 1, G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, CHART_PROP_ROWS,
+		g_param_spec_int ("rows", _("rows"),
+			_("Number of rows in graph grid"),
+			1, G_MAXINT, 1, G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_klass->view_type = gog_chart_view_get_type ();
 	gog_klass->update    = gog_chart_update;
@@ -508,10 +570,12 @@ gog_chart_class_init (GogObjectClass *gog_klass)
 static void
 gog_chart_init (GogChart *chart)
 {
-	chart->x     = 0;
-	chart->y     = 0;
-	chart->cols  = 0;
-	chart->rows  = 0;
+	chart->x_pos =
+	chart->y_pos = 
+	chart->cols  = 		
+	chart->rows  =
+	chart->x_pos_actual = 
+	chart->y_pos_actual = 0;
 
 	/* start as true so that we can queue an update when it changes */
 	chart->cardinality_valid = TRUE;
@@ -520,7 +584,7 @@ gog_chart_init (GogChart *chart)
 	chart->is_plot_area_manual = FALSE;
 	chart->plot_area.x =
 	chart->plot_area.y = 0.0;
-	chart->plot_area.w = 
+	chart->plot_area.w =
 	chart->plot_area.h = 1.0;
 }
 
@@ -547,8 +611,8 @@ gog_chart_get_position (GogChart const *chart,
 	if (chart->cols <= 0 || chart->rows <= 0)
 		return FALSE;
 
-	if (x != NULL)	  *x	= chart->x;
-	if (y != NULL)	  *y	= chart->y;
+	if (x != NULL)	  *x	= chart->x_pos;
+	if (y != NULL)	  *y	= chart->y_pos;
 	if (cols != NULL) *cols	= chart->cols;
 	if (rows != NULL) *rows	= chart->rows;
 
@@ -570,12 +634,12 @@ gog_chart_set_position (GogChart *chart,
 {
 	g_return_if_fail (GOG_IS_CHART (chart));
 
-	if (chart->x == x && chart->y == y &&
+	if (chart->x_pos == x && chart->y_pos == y &&
 	    chart->cols == cols && chart->rows == rows)
 		return;
 
-	chart->x = x;
-	chart->y = y;
+	chart->x_pos = x;
+	chart->y_pos = y;
 	chart->cols = cols;
 	chart->rows = rows;
 
@@ -597,7 +661,7 @@ gog_chart_get_plot_area (GogChart *chart, GogViewAllocation *plot_area)
 {
 	if (plot_area != NULL)
 		*plot_area = chart->plot_area;
-	
+
 	return chart->is_plot_area_manual;
 }
 
@@ -629,12 +693,12 @@ gog_chart_set_plot_area (GogChart *chart, GogViewAllocation const *plot_area)
  * @visible: placeholder for visible cardinality
  *
  * Update and cache cardinality values if required, and returns
- * full and visible cardinality. Full cardinality is the number of 
+ * full and visible cardinality. Full cardinality is the number of
  * chart elements that require a different style. Visible cardinality is
  * the number of chart elements shown in chart legend.
  *
  * @full and @visible may be NULL.
- **/ 
+ **/
 void
 gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 {
@@ -664,7 +728,7 @@ void
 gog_chart_request_cardinality_update (GogChart *chart)
 {
 	g_return_if_fail (GOG_IS_CHART (chart));
-	
+
 	if (chart->cardinality_valid) {
 		chart->cardinality_valid = FALSE;
 		gog_object_request_update (GOG_OBJECT (chart));
@@ -968,7 +1032,7 @@ gog_chart_view_3d_process (GogView *view, GogViewAllocation *bbox)
 	box->ratio = (d > tg)? d: tg;
 
 	gog_view_padding_request (view, bbox, &padding);
-	
+
 	if (!chart->is_plot_area_manual) {
 		bbox->x += padding.wl;
 		bbox->w -= padding.wl + padding.wr;
@@ -1042,7 +1106,7 @@ gog_chart_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 
 	tmp = *plot_area;
 	gog_view_padding_request (view, plot_area, &padding);
-	
+
 	if (!chart->is_plot_area_manual) {
 		plot_area->x += padding.wl;
 		plot_area->w -= padding.wl + padding.wr;
@@ -1076,7 +1140,7 @@ gog_chart_view_init (GogChartView *cview)
 }
 
 static void
-grid_line_render (GSList *start_ptr, GogViewAllocation const *bbox) 
+grid_line_render (GSList *start_ptr, GogViewAllocation const *bbox)
 {
 	GSList *ptr, *child_ptr;
 	GSList *minor_grid_lines = NULL;
@@ -1090,16 +1154,16 @@ grid_line_render (GSList *start_ptr, GogViewAllocation const *bbox)
 				axis_child_view = child_ptr->data;
 				if (GOG_IS_GRID_LINE (axis_child_view->model)) {
 					if (gog_grid_line_is_minor (GOG_GRID_LINE (axis_child_view->model)))
-						minor_grid_lines = g_slist_prepend (minor_grid_lines, 
+						minor_grid_lines = g_slist_prepend (minor_grid_lines,
 										    axis_child_view);
 					else
-						major_grid_lines = g_slist_prepend (major_grid_lines, 
+						major_grid_lines = g_slist_prepend (major_grid_lines,
 										    axis_child_view);
-				}	
+				}
 			}
 		}
-	}	    
-	
+	}
+
 	/* Render stripes, minor first then major */
 	for (ptr = minor_grid_lines; ptr != NULL; ptr = ptr->next) {
 		gog_grid_line_view_render_stripes (ptr->data);
@@ -1107,7 +1171,7 @@ grid_line_render (GSList *start_ptr, GogViewAllocation const *bbox)
 	for (ptr = major_grid_lines; ptr != NULL; ptr = ptr->next) {
 		gog_grid_line_view_render_stripes (ptr->data);
 	}
-	
+
 	/* Render lines, minor first then major */
 	for (ptr = minor_grid_lines; ptr != NULL; ptr = ptr->next) {
 		gog_grid_line_view_render_lines (ptr->data);
@@ -1115,13 +1179,13 @@ grid_line_render (GSList *start_ptr, GogViewAllocation const *bbox)
 	for (ptr = major_grid_lines; ptr != NULL; ptr = ptr->next) {
 		gog_grid_line_view_render_lines (ptr->data);
 	}
-	
+
 	g_slist_free (minor_grid_lines);
 	g_slist_free (major_grid_lines);
 }
 
 static void
-plot_render (GogView *view, GogViewAllocation const *bbox) 
+plot_render (GogView *view, GogViewAllocation const *bbox)
 {
 	GSList *ptr;
 	GogView *child_view;
@@ -1129,7 +1193,7 @@ plot_render (GogView *view, GogViewAllocation const *bbox)
 	/* Render some plots before axes */
 	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
 		child_view = ptr->data;
-		if (GOG_IS_PLOT (child_view->model) && 
+		if (GOG_IS_PLOT (child_view->model) &&
 		    GOG_PLOT (child_view->model)->render_before_axes)
 			gog_view_render	(ptr->data, bbox);
 	}
@@ -1148,7 +1212,7 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 	if (gog_chart_is_3d (chart)) {
 		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
 			child_view = ptr->data;
-			if (!GOG_IS_AXIS (child_view->model) && !GOG_IS_PLOT (child_view->model)) 
+			if (!GOG_IS_AXIS (child_view->model) && !GOG_IS_PLOT (child_view->model) && !GOG_IS_LABEL (child_view->model))
 				gog_view_render	(ptr->data, bbox);
 		}
 		/* now render plot and axes */
@@ -1161,10 +1225,9 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 		}
 		for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
 			child_view = ptr->data;
-			if (GOG_IS_AXIS (child_view->model))
+			if (!GOG_IS_PLOT (child_view->model))
 				continue;
-			if (GOG_IS_PLOT (child_view->model)) 
-				gog_view_render	(ptr->data, bbox);
+			gog_view_render	(ptr->data, bbox);
 		}
 	} else {
 		/* KLUDGE: render grid lines before axis */
@@ -1178,10 +1241,16 @@ gog_chart_view_render (GogView *view, GogViewAllocation const *bbox)
 			if (GOG_IS_PLOT (child_view->model)) {
 			    if (!GOG_PLOT (child_view->model)->render_before_axes)
 				gog_view_render	(ptr->data, bbox);
-			} else
+			} else if (!GOG_IS_LABEL (child_view->model))
 				gog_view_render	(ptr->data, bbox);
 		}
 	}
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next) {
+		child_view = ptr->data;
+		if (!GOG_IS_LABEL (child_view->model))
+			continue;
+		gog_view_render	(ptr->data, bbox);
+	}
 }
 
 static void
