@@ -59,8 +59,9 @@ gog_probability_plot_populate_editor (GogObject *item,
 			      GogDataAllocator *dalloc,
 			      GOCmdContext *cc)
 {
-	go_editor_add_page (editor, go_distribution_pref_new (G_OBJECT (item), dalloc, cc), _("Distribution"));
-	
+	GtkWidget *w = go_distribution_pref_new (G_OBJECT (item), dalloc, cc);
+	go_editor_add_page (editor, w, _("Distribution"));
+
 	(GOG_OBJECT_CLASS(probability_plot_parent_klass)->populate_editor) (item, editor, dalloc, cc);
 }
 #endif
@@ -69,20 +70,34 @@ static void
 gog_probability_plot_update (GogObject *obj)
 {
 	GogProbabilityPlot *plot = GOG_PROBABILITY_PLOT (obj);
-	GogProbabilityPlotSeries *series = GOG_PROBABILITY_PLOT_SERIES (
-		plot->base.series->data);
+	GogProbabilityPlotSeries *series;
+	double x_min, x_max, y_min, y_max;
+	GSList *ptr;
 
+	x_min = y_min =  DBL_MAX;
+	x_max = y_max = -DBL_MAX;
+	for (ptr = plot->base.series ; ptr != NULL ; ptr = ptr->next) {
+		series = GOG_PROBABILITY_PLOT_SERIES (ptr->data);
+		if (!gog_series_is_valid (GOG_SERIES (series)) || series->base.num_elements == 0)
+			continue;
 
-	if (!gog_series_is_valid (GOG_SERIES (series)) || series->base.num_elements == 0)
-			return;
-	if (plot->x.minima != series->x[0] || plot->x.maxima != series->x[series->base.num_elements - 1]) {
-		plot->x.minima = series->x[0];
-		plot->x.maxima = series->x[series->base.num_elements - 1];
+		if (x_min > series->x[0])
+			x_min = series->x[0];
+		if (x_max < series->x[series->base.num_elements - 1])
+			x_max = series->x[series->base.num_elements - 1];
+		if (y_min > series->y[0])
+			y_min = series->y[0];
+		if (y_max < series->y[series->base.num_elements - 1])
+			y_max = series->y[series->base.num_elements - 1];
+	}
+	if (plot->x.minima != x_min || plot->x.maxima != x_max) {
+		plot->x.minima = x_min;
+		plot->x.maxima = x_max;
 		gog_axis_bound_changed (plot->base.axis[0], GOG_OBJECT (plot));
 	}
-	if (plot->y.minima != series->y[0] || plot->y.maxima != series->y[series->base.num_elements - 1]) {
-		plot->y.minima = series->y[0];
-		plot->y.maxima = series->y[series->base.num_elements - 1];
+	if (plot->y.minima != y_min || plot->y.maxima != y_max) {
+		plot->y.minima = y_min;
+		plot->y.maxima = y_max;
 		gog_axis_bound_changed (plot->base.axis[1], GOG_OBJECT (plot));
 	}
 }
@@ -234,22 +249,22 @@ gog_probability_plot_class_init (GogPlotClass *gog_plot_klass)
 	gobject_klass->finalize = gog_probability_plot_finalize;
 
 	g_object_class_install_property (gobject_klass, PROBABILITY_PLOT_PROP_DISTRIBUTION,
-		g_param_spec_object ("distribution", 
+		g_param_spec_object ("distribution",
 			_("Distribution"),
 			_("A pointer to the GODistribution used by this plot"),
-			GO_TYPE_DISTRIBUTION, 
+			GO_TYPE_DISTRIBUTION,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PROBABILITY_PLOT_PROP_SHAPE_PARAM1,
-		g_param_spec_string ("param1", 
+		g_param_spec_string ("param1",
 			_("Shape parameter"),
 			_("Name of the first shape parameter if any"),
-			"none", 
+			"none",
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PROBABILITY_PLOT_PROP_SHAPE_PARAM2,
-		g_param_spec_string ("param2", 
+		g_param_spec_string ("param2",
 			_("Second shape parameter"),
 			_("Name of the second shape parameter if any"),
-			"none", 
+			"none",
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_object_klass->type_name	= gog_probability_plot_type_name;
@@ -264,7 +279,7 @@ gog_probability_plot_class_init (GogPlotClass *gog_plot_klass)
 		plot_klass->desc.series.dim = dimensions;
 		plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
 	}
-	plot_klass->desc.num_series_max = 1;
+	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type = gog_probability_plot_series_get_type ();
 	plot_klass->axis_set = GOG_AXIS_SET_XY;
 	plot_klass->desc.series.style_fields	= GO_STYLE_MARKER;
@@ -359,17 +374,17 @@ gog_probability_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogProbabilityPlotSeries const *series;
 	unsigned i, nb;
 	GOStyle *style;
+	GSList *ptr;
 
 	if (model->base.series == NULL)
 		return;
-	series = GOG_PROBABILITY_PLOT_SERIES (model->base.series->data);
-	nb = series->base.num_elements;
-	style = GOG_STYLED_OBJECT (series)->style;
-	if (nb == 0 || series->x == NULL || series->y == NULL)
-		return;
+
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
+		gog_view_render	(ptr->data, bbox);
+	
 	area = gog_chart_view_get_plot_area (view->parent);
-	chart_map = gog_chart_map_new (chart, area, 
-				       GOG_PLOT (model)->axis[GOG_AXIS_X], 
+	chart_map = gog_chart_map_new (chart, area,
+				       GOG_PLOT (model)->axis[GOG_AXIS_X],
 				       GOG_PLOT (model)->axis[GOG_AXIS_Y],
 				       NULL, FALSE);
 	if (!gog_chart_map_is_valid (chart_map)) {
@@ -380,23 +395,42 @@ gog_probability_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 	x_map = gog_chart_map_get_axis_map (chart_map, 0);
 	y_map = gog_chart_map_get_axis_map (chart_map, 1);
 
-	gog_renderer_push_style (view->renderer, style);
-	for (i = 0; i < nb; i++) {
-		gog_renderer_draw_marker (view->renderer,
-					  gog_axis_map_to_view (x_map, series->x[i]),
-					  gog_axis_map_to_view (y_map, series->y[i]));
+	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
+		series = GOG_PROBABILITY_PLOT_SERIES (ptr->data);
+		if (!gog_series_is_valid (GOG_SERIES (series))
+		    || series->base.num_elements == 0
+		    || series->x == NULL || series->y == NULL)
+			continue;
+		nb = series->base.num_elements;
+		style = GOG_STYLED_OBJECT (series)->style;
+		gog_renderer_push_style (view->renderer, style);
+		for (i = 0; i < nb; i++) {
+			gog_renderer_draw_marker (view->renderer,
+						  gog_axis_map_to_view (x_map, series->x[i]),
+						  gog_axis_map_to_view (y_map, series->y[i]));
+		}
+		gog_renderer_pop_style (view->renderer);
 	}
-	gog_renderer_pop_style (view->renderer);
 	gog_chart_map_free (chart_map);
 }
 
 static GogViewClass *probability_plot_view_parent_klass;
 
+static void
+gog_probability_plot_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
+{
+	GSList *ptr;
+	for (ptr = view->children; ptr != NULL; ptr = ptr->next)
+		gog_view_size_allocate (GOG_VIEW (ptr->data), allocation);
+	(probability_plot_view_parent_klass->size_allocate) (view, allocation);
+}
+
 static void
 gog_probability_plot_view_class_init (GogViewClass *view_klass)
 {
 	probability_plot_view_parent_klass = (GogViewClass*) g_type_class_peek_parent (view_klass);
 	view_klass->render	  = gog_probability_plot_view_render;
+	view_klass->size_allocate = gog_probability_plot_view_size_allocate;
 	view_klass->clip	  = FALSE;
 }
 
@@ -404,6 +438,66 @@ GSF_DYNAMIC_CLASS (GogProbabilityPlotView, gog_probability_plot_view,
 	gog_probability_plot_view_class_init, NULL,
 	GOG_TYPE_PLOT_VIEW)
 
+
+/*****************************************************************************/
+
+static gboolean
+regression_curve_can_add (GogObject const *parent)
+{
+	return (gog_object_get_child_by_name (parent, "Regression line") == NULL);
+}
+
+static void
+regression_curve_post_add (GogObject *parent, GogObject *child)
+{
+	gog_object_request_update (child);
+}
+
+static void
+regression_curve_pre_remove (GogObject *parent, GogObject *child)
+{
+}
+
+/*****************************************************************************/
+
+typedef GogView		GogProbabilityPlotSeriesView;
+typedef GogViewClass	GogProbabilityPlotSeriesViewClass;
+
+#define GOG_TYPE_PROBABILITY_SERIES_VIEW	(gog_probability_plot_series_view_get_type ())
+#define GOG__PROBABILITY_SERIES_VIEW(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_TYPE__PROBABILITY_SERIES_VIEW, GogProbabilityPlotSeriesView))
+#define GOG_IS__PROBABILITY_SERIES_VIEW(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_TYPE__PROBABILITY_SERIES_VIEW))
+
+GType gog_probability_plot_series_view_get_type (void);
+
+static void
+gog_probability_plot_series_view_render (GogView *view, GogViewAllocation const *bbox)
+{
+	GSList *ptr;
+	for (ptr = view->children ; ptr != NULL ; ptr = ptr->next)
+		gog_view_render	(ptr->data, bbox);
+}
+
+static void
+gog_probability_plot_series_view_size_allocate (GogView *view, GogViewAllocation const *allocation)
+{
+	GSList *ptr;
+
+	for (ptr = view->children; ptr != NULL; ptr = ptr->next)
+		gog_view_size_allocate (GOG_VIEW (ptr->data), allocation);
+}
+
+static void
+gog_probability_plot_series_view_class_init (GogProbabilityPlotSeriesViewClass *gview_klass)
+{
+	GogViewClass *view_klass = GOG_VIEW_CLASS (gview_klass);
+	view_klass->render = gog_probability_plot_series_view_render;
+	view_klass->size_allocate = gog_probability_plot_series_view_size_allocate;
+	view_klass->build_toolkit = NULL;
+}
+
+GSF_DYNAMIC_CLASS (GogProbabilityPlotSeriesView, gog_probability_plot_series_view,
+	gog_probability_plot_series_view_class_init, NULL,
+	GOG_TYPE_VIEW)
 /****************************************************************************/
 
 static GogObjectClass *gog_probability_plot_series_parent_klass;
@@ -418,6 +512,7 @@ gog_probability_plot_series_update (GogObject *obj)
 	double mn, d;
 	unsigned i;
 	GODistribution *dist = GO_DISTRIBUTION (((GogProbabilityPlot *) series->base.plot)->dist);
+	GSList *ptr;
 
 	g_free (series->x);
 	series->x = NULL;
@@ -442,6 +537,11 @@ gog_probability_plot_series_update (GogObject *obj)
 	} else
 		series->y = NULL;
 
+	/* update children */
+	for (ptr = obj->children; ptr != NULL; ptr = ptr->next)
+		if (!GOG_IS_SERIES_LINES (ptr->data))
+			gog_object_request_update (GOG_OBJECT (ptr->data));
+
 	/* queue plot for redraw */
 	gog_object_request_update (GOG_OBJECT (series->base.plot));
 
@@ -449,6 +549,17 @@ gog_probability_plot_series_update (GogObject *obj)
 		gog_probability_plot_series_parent_klass->update (obj);
 }
 
+static unsigned
+gog_probability_plot_series_get_xy_data (GogSeries const  *series,
+			double const 	**x,
+			double const 	**y)
+{
+	GogProbabilityPlotSeries *ppseries = GOG_PROBABILITY_PLOT_SERIES (series);
+	*x = ppseries->x;
+	*y = ppseries->y;
+	return series->num_elements;
+}
+
 static void
 gog_probability_plot_series_finalize (GObject *obj)
 {
@@ -466,12 +577,28 @@ static void
 gog_probability_plot_series_class_init (GogObjectClass *obj_klass)
 {
 	GObjectClass *gobject_klass = (GObjectClass *) obj_klass;
+	GogSeriesClass *series_klass = (GogSeriesClass *) obj_klass;
+
+	static GogObjectRole const roles[] = {
+		{ N_("Regression line"), "GogLinRegCurve",	0,
+		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+		  regression_curve_can_add,
+		  NULL,
+		  NULL,
+		  regression_curve_post_add,
+		  regression_curve_pre_remove,
+		  NULL }
+	};
 
 	series_parent_klass = g_type_class_peek_parent (obj_klass);
 	gobject_klass->finalize		= gog_probability_plot_series_finalize;
 
 	gog_probability_plot_series_parent_klass = g_type_class_peek_parent (obj_klass);
 	obj_klass->update = gog_probability_plot_series_update;
+	obj_klass->view_type	= gog_probability_plot_series_view_get_type ();
+	gog_object_register_roles (obj_klass, roles, G_N_ELEMENTS (roles));
+
+	series_klass->get_xy_data = gog_probability_plot_series_get_xy_data;
 }
 
 GSF_DYNAMIC_CLASS (GogProbabilityPlotSeries, gog_probability_plot_series,
