@@ -35,6 +35,7 @@
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 #include <math.h>
+#include <string.h>
 
 static GObjectClass *ppe_parent_klass;
 
@@ -51,7 +52,7 @@ gog_pie_series_element_set_property (GObject *obj, guint param_id,
 	GogPieSeriesElement *pse = GOG_PIE_SERIES_ELEMENT (obj);
 
 	switch (param_id) {
-	case ELEMENT_SEPARATION :
+	case ELEMENT_SEPARATION:
 		pse->separation = g_value_get_float (value);
 		break;
 
@@ -69,7 +70,7 @@ gog_pie_series_element_get_property (GObject *obj, guint param_id,
 	GogPieSeriesElement *pse = GOG_PIE_SERIES_ELEMENT (obj);
 
 	switch (param_id) {
-	case ELEMENT_SEPARATION :
+	case ELEMENT_SEPARATION:
 		g_value_set_float (value, pse->separation);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -98,14 +99,14 @@ gog_pie_series_element_class_init (GogPieSeriesElementClass *klass)
 
 	gobject_klass->set_property = gog_pie_series_element_set_property;
 	gobject_klass->get_property = gog_pie_series_element_get_property;
-	
+
 	ppe_parent_klass 	= g_type_class_peek_parent (klass);
 #ifdef GOFFICE_WITH_GTK
 	klass->gse_populate_editor	= gog_pie_series_element_populate_editor;
 #endif
 
 	g_object_class_install_property (gobject_klass, ELEMENT_SEPARATION,
-		g_param_spec_float ("separation", 
+		g_param_spec_float ("separation",
 			_("Separation"),
 			_("Amount a slice is extended as a percentage of the radius"),
 			0, 1000, 0.,
@@ -118,6 +119,54 @@ GSF_DYNAMIC_CLASS (GogPieSeriesElement, gog_pie_series_element,
 
 /*****************************************************************************/
 
+typedef enum {
+	GOG_SHOW_NEGS_SKIP,
+	GOG_SHOW_NEGS_ABSOLUTE,
+	GOG_SHOW_NEGS_WHITE,
+	GOG_SHOW_NEGS_INVERTED, /* not used for now */
+	GOG_SHOW_NEGS_MAX
+} GogShowNegsMode;
+
+static struct {
+	GogShowNegsMode mode;
+	char const *name;
+} neg_modes [GOG_SHOW_NEGS_MAX] = {
+	{GOG_SHOW_NEGS_SKIP, "skip"},
+	{GOG_SHOW_NEGS_ABSOLUTE, "absolute"},
+	{GOG_SHOW_NEGS_WHITE, "white"},
+	{GOG_SHOW_NEGS_INVERTED, "inverted"}
+};
+
+static GogShowNegsMode
+gog_show_neg_mode_from_str (char const *name)
+{
+	unsigned i;
+	GogShowNegsMode ret = GOG_SHOW_NEGS_ABSOLUTE;
+
+	for (i = 0; i < GO_LINE_MAX; i++) {
+		if (strcmp (neg_modes[i].name, name) == 0) {
+			ret = neg_modes[i].mode;
+			break;
+		}
+	}
+	return ret;
+}
+
+static char const *
+gog_show_neg_mode_as_str (GogShowNegsMode mode)
+{
+	unsigned i;
+	char const *ret = "absolute";
+
+	for (i = 0; i < GOG_SHOW_NEGS_MAX; i++) {
+		if (neg_modes[i].mode == mode) {
+			ret = neg_modes[i].name;
+			break;
+		}
+	}
+	return ret;
+}
+
 typedef struct {
 	GogPlotClass	base;
 } GogPiePlotClass;
@@ -127,7 +176,8 @@ enum {
 	PLOT_PROP_INITIAL_ANGLE,
 	PLOT_PROP_DEFAULT_SEPARATION,
 	PLOT_PROP_IN_3D,
-	PLOT_PROP_SPAN
+	PLOT_PROP_SPAN,
+	PLOT_PROP_SHOW_NEGS
 };
 
 GOFFICE_PLUGIN_MODULE_HEADER;
@@ -140,22 +190,32 @@ gog_pie_plot_set_property (GObject *obj, guint param_id,
 			   GValue const *value, GParamSpec *pspec)
 {
 	GogPiePlot *pie = GOG_PIE_PLOT (obj);
-	float f;
 
 	switch (param_id) {
-	case PLOT_PROP_INITIAL_ANGLE :
+	case PLOT_PROP_INITIAL_ANGLE:
 		pie->initial_angle = g_value_get_float (value);
 		break;
-	case PLOT_PROP_DEFAULT_SEPARATION :
-		f = g_value_get_float (value);
+	case PLOT_PROP_DEFAULT_SEPARATION: {
+		double f = g_value_get_float (value);
 		pie->default_separation = CLAMP (f, 0., 5.);
 		break;
-	case PLOT_PROP_IN_3D :
+	}
+	case PLOT_PROP_IN_3D:
 		pie->in_3d = g_value_get_boolean (value);
 		break;
-	case PLOT_PROP_SPAN :
+	case PLOT_PROP_SPAN:
 		pie->span = g_value_get_float (value);
 		break;
+	case PLOT_PROP_SHOW_NEGS : {
+		GSList *ptr = GOG_PLOT (obj)->series;
+		pie->show_negatives = gog_show_neg_mode_from_str (g_value_get_string (value));
+		/* we need to update all the series */
+		while (ptr) {
+			gog_object_request_update (GOG_OBJECT (ptr->data));
+			ptr = ptr->next;
+		}
+		break;
+	}
 
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 return; /* NOTE : RETURN */
@@ -173,18 +233,21 @@ gog_pie_plot_get_property (GObject *obj, guint param_id,
 	GogPiePlot *pie = GOG_PIE_PLOT (obj);
 
 	switch (param_id) {
-	case PLOT_PROP_INITIAL_ANGLE :
+	case PLOT_PROP_INITIAL_ANGLE:
 		g_value_set_float (value, pie->initial_angle);
 		break;
-	case PLOT_PROP_DEFAULT_SEPARATION :
+	case PLOT_PROP_DEFAULT_SEPARATION:
 		g_value_set_float (value, pie->default_separation);
 		break;
-	case PLOT_PROP_IN_3D :
+	case PLOT_PROP_IN_3D:
 		g_value_set_boolean (value, pie->in_3d);
 		break;
-	case PLOT_PROP_SPAN :
+	case PLOT_PROP_SPAN:
 		g_value_set_float (value, pie->span);
 		break;
+	case PLOT_PROP_SHOW_NEGS:
+		g_value_set_string (value, gog_show_neg_mode_as_str (pie->show_negatives));
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -196,21 +259,23 @@ gog_pie_plot_type_name (G_GNUC_UNUSED GogObject const *item)
 	return N_("PlotPie");
 }
 
+#ifdef GOFFICE_WITH_GTK
 extern gpointer gog_pie_plot_pref (GogPiePlot *pie, GOCmdContext *cc);
 static void
-gog_pie_plot_populate_editor (GogObject *item, 
+gog_pie_plot_populate_editor (GogObject *item,
 			      GOEditor *editor,
 			      G_GNUC_UNUSED GogDataAllocator *dalloc,
 			      GOCmdContext *cc)
 {
 	GtkWidget *widget = gog_pie_plot_pref (GOG_PIE_PLOT (item), cc);
-	go_editor_add_page (editor, 
+	go_editor_add_page (editor,
 			     widget,
 			     _("Properties"));
 	g_object_unref (G_OBJECT (widget));
 
 	(GOG_OBJECT_CLASS(pie_parent_klass)->populate_editor) (item, editor, dalloc, cc);
 }
+#endif
 
 static void
 gog_pie_plot_update (GogObject *obj)
@@ -230,33 +295,41 @@ gog_pie_plot_class_init (GogPlotClass *plot_klass)
 
 	gog_klass->update	= gog_pie_plot_update;
 	gog_klass->type_name	= gog_pie_plot_type_name;
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor = gog_pie_plot_populate_editor;
+#endif
 	gog_klass->view_type	= gog_pie_view_get_type ();
 
 	g_object_class_install_property (gobject_klass, PLOT_PROP_INITIAL_ANGLE,
-		g_param_spec_float ("initial-angle", 
+		g_param_spec_float ("initial-angle",
 			_("Initial angle"),
 			_("Degrees clockwise from 12 O'Clock."),
 			0, G_MAXFLOAT, 0.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_DEFAULT_SEPARATION,
-		g_param_spec_float ("default-separation", 
+		g_param_spec_float ("default-separation",
 			_("Default separation"),
 			_("Default amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_IN_3D,
-		g_param_spec_boolean ("in-3d", 
+		g_param_spec_boolean ("in-3d",
 			_("In 3d"),
 			_("Draw 3d wedges"),
 			FALSE,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_SPAN,
-		g_param_spec_float ("span", 
+		g_param_spec_float ("span",
 			_("Span"),
 			_("Total angle used as a percentage of the full circle"),
 			10., 100., 100.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, PLOT_PROP_SHOW_NEGS,
+		g_param_spec_string ("show-negs",
+			_("Show negative values"),
+			_("How negative values are displayed"),
+			"absolute",
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	{
 		static GogSeriesDimDesc dimensions[] = {
@@ -278,6 +351,7 @@ gog_pie_plot_init (GogPiePlot *pie)
 {
 	pie->base.vary_style_by_element = TRUE;
 	pie->span = 100.;
+	pie->show_negatives = GOG_SHOW_NEGS_ABSOLUTE;
 }
 
 GSF_DYNAMIC_CLASS (GogPiePlot, gog_pie_plot,
@@ -289,7 +363,7 @@ gog_pie_plot_set_default_separation (GogPiePlot *pie, double separation)
 {
 	g_return_if_fail (GOG_PIE_PLOT (pie) != NULL);
 
-	pie->default_separation = CLAMP (separation, 0.0, 5.0); 
+	pie->default_separation = CLAMP (separation, 0.0, 5.0);
 	gog_object_emit_changed (GOG_OBJECT (pie), FALSE);
 }
 
@@ -310,7 +384,7 @@ gog_ring_plot_set_property (GObject *obj, guint param_id,
 	GogRingPlot *ring = GOG_RING_PLOT (obj);
 
 	switch (param_id) {
-	case RING_PLOT_PROP_CENTER_SIZE :
+	case RING_PLOT_PROP_CENTER_SIZE:
 		ring->center_size = g_value_get_float (value);
 		break;
 
@@ -330,7 +404,7 @@ gog_ring_plot_get_property (GObject *obj, guint param_id,
 	GogRingPlot *ring = GOG_RING_PLOT (obj);
 
 	switch (param_id) {
-	case RING_PLOT_PROP_CENTER_SIZE :
+	case RING_PLOT_PROP_CENTER_SIZE:
 		g_value_set_float (value, ring->center_size);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -344,6 +418,7 @@ gog_ring_plot_type_name (G_GNUC_UNUSED GogObject const *item)
 	return N_("PlotRing");
 }
 
+#ifdef GOFFICE_WITH_GTK
 extern gpointer gog_ring_plot_pref (GogRingPlot *ring, GOCmdContext *cc);
 static void
 gog_ring_plot_populate_editor (GogObject *item,
@@ -357,6 +432,7 @@ gog_ring_plot_populate_editor (GogObject *item,
 			     _("Properties"));
 	g_object_unref (G_OBJECT (widget));
 }
+#endif
 
 static void
 gog_ring_plot_class_init (GogPiePlotClass *pie_plot_klass)
@@ -370,10 +446,12 @@ gog_ring_plot_class_init (GogPiePlotClass *pie_plot_klass)
 	gobject_klass->get_property = gog_ring_plot_get_property;
 
 	gog_klass->type_name	= gog_ring_plot_type_name;
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor = gog_ring_plot_populate_editor;
+#endif
 
 	g_object_class_install_property (gobject_klass, RING_PLOT_PROP_CENTER_SIZE,
-		g_param_spec_float ("center-size", 
+		g_param_spec_float ("center-size",
 			_("Center-size"),
 			_("Size of the center hole as a percentage of the radius"),
 			0, 100., 0.,
@@ -403,7 +481,7 @@ typedef struct {
 } MovePieData;
 
 static gboolean
-find_element (GogView *view, double cx, double cy, double x, double y, 
+find_element (GogView *view, double cx, double cy, double x, double y,
 	      unsigned int *index, GogPieSeries **series)
 {
 	GogPiePlot *pie = GOG_PIE_PLOT (view->model);
@@ -419,7 +497,7 @@ find_element (GogView *view, double cx, double cy, double x, double y,
 	if (ptr == NULL)
 		return FALSE;
 
-	theta = (atan2 (y - cy, x - cx) 
+	theta = (atan2 (y - cy, x - cx)
 		 * 180 / M_PI - pie->initial_angle + 90.) / pie->span / 3.6;
 	if (theta < 0)
 		theta += 1.;
@@ -427,7 +505,9 @@ find_element (GogView *view, double cx, double cy, double x, double y,
 	vals = go_data_get_values ((*series)->base.values[1].data);
 	scale = 1 / (*series)->total;
 	for (*index = 0 ; *index < (*series)->base.num_elements; (*index)++) {
-		len = fabs (vals[*index]) * scale;
+		len = vals[*index] * scale;
+		if (len < 0.)
+			len = pie->show_negatives? -len: 0.;
 		if (go_finite (len) && len > 1e-3) {
 			theta -= len;
 			if (theta < 0)
@@ -443,7 +523,7 @@ gog_tool_move_pie_point (GogView *view, double x, double y, GogObject **gobj)
 	GogPieSeries *series;
 	double r = view->allocation.h, cx, cy;
 	unsigned int index;
-	
+
 	if (r > view->allocation.w)
 		r = view->allocation.w;
 	r /= 2.;
@@ -452,8 +532,8 @@ gog_tool_move_pie_point (GogView *view, double x, double y, GogObject **gobj)
 
 	if (hypot (x - cx, y - cy) > fabs (r))
 		return FALSE;
-	
-	if (find_element (view, cx, cy, x, y, &index, &series)) 
+
+	if (find_element (view, cx, cy, x, y, &index, &series))
 		*gobj = GOG_OBJECT (gog_series_get_element (GOG_SERIES (series), index));
 
 	return TRUE;
@@ -502,12 +582,12 @@ gog_tool_move_pie_move (GogToolAction *action, double x, double y)
 	MovePieData *data = action->data;
 	double pos, separation;
 
-	pos = data->start_pos - 
+	pos = data->start_pos -
 		((x - action->start_x) * (data->x - action->start_x) +
-		 (y - action->start_y) * (data->y - action->start_y)) / 
-		data->start_distance; 
+		 (y - action->start_y) * (data->y - action->start_y)) /
+		data->start_distance;
 	separation = (pos - 0.5 * data->r) / (data->r - pos);
-					     
+
 	gog_pie_plot_set_default_separation (pie, separation);
 }
 
@@ -519,11 +599,11 @@ gog_tool_move_pie_double_click (GogToolAction *action)
 	GogObject *obj;
 	unsigned int index;
 
-	if (!find_element (action->view, data->x, data->y, 
+	if (!find_element (action->view, data->x, data->y,
 			   action->start_x, action->start_y,
 			   &index, &series))
 		return;
-	
+
 	obj = GOG_OBJECT (gog_series_get_element (GOG_SERIES (series), index));
 	if (obj == NULL) {
 		obj = g_object_new (gog_pie_series_element_get_type (),
@@ -535,10 +615,10 @@ gog_tool_move_pie_double_click (GogToolAction *action)
 static GogTool gog_tool_move_pie = {
 	N_("Move pie"),
 	GDK_FLEUR,
-	gog_tool_move_pie_point, 
+	gog_tool_move_pie_point,
 	gog_tool_move_pie_render,
-	gog_tool_move_pie_init, 
-	gog_tool_move_pie_move, 
+	gog_tool_move_pie_init,
+	gog_tool_move_pie_move,
 	gog_tool_move_pie_double_click,
 	NULL /*destroy*/
 };
@@ -551,6 +631,77 @@ typedef GogPlotViewClass	GogPieViewClass;
 
 static GogViewClass *pie_view_parent_klass;
 
+static int
+gog_pie_view_get_data_at_point (GogPlotView *view, double x, double y, GogSeries **series)
+{
+	GogPiePlot const *model = GOG_PIE_PLOT (view->base.model);
+	double r_max = view->base.allocation.h, cx, cy, r, h, theta, scale, *vals;
+	unsigned int index;
+	double center_size = 0.0;
+	unsigned num_series = 0;
+	double default_sep;
+	GSList *ptr;
+
+	/* compute number of valid series */
+	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
+		*series = ptr->data;
+		if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+			continue;
+		num_series++;
+	}
+
+	if (num_series <= 0)
+		return -1;
+
+	if (r_max > view->base.allocation.w)
+		r_max = view->base.allocation.w;
+	r_max /= 2.;
+	cx = view->base.allocation.x + view->base.allocation.w/2.;
+	cy = view->base.allocation.y + view->base.allocation.h/2.;
+	r = hypot (x - cx, y - cy);
+
+	if (r > r_max)
+		return -1;
+
+	if (GOG_IS_RING_PLOT (model))
+		center_size = GOG_RING_PLOT(model)->center_size;
+	/* FIXME: this is somewhat approximative, must be enhanced for ring plots with a slice separation */
+	center_size *= r_max;
+	if (r < center_size)
+		return -1;
+
+	default_sep = r_max * model->default_separation;
+	h = (r_max + default_sep - center_size) / num_series;
+	index = floor ((r - center_size) / h);
+	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
+		*series = ptr->data;
+		if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+			continue;
+		if (index-- == 0)
+			break;
+	}
+	theta = (atan2 (y - cy, x - cx)
+		 * 180 / M_PI - model->initial_angle + 90.) / model->span / 3.6;
+	if (theta < 0)
+		theta += 1.;
+
+	vals = go_data_get_values ((*series)->values[1].data);
+	if (!vals)
+		return -1;
+	scale = 1 / GOG_PIE_SERIES (*series)->total;
+	for (index = 0 ; index < (*series)->num_elements; index++) {
+		r = vals[index] * scale;
+		if (r < 0.)
+			r = model->show_negatives? -r: 0.;
+		if (go_finite (r) && r > 1e-3) {
+			theta -= r;
+			if (theta < 0)
+				break;
+		}
+	}
+	return (int) index;
+}
+
 #define MAX_ARC_SEGMENTS 64
 
 static void
@@ -563,7 +714,7 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	unsigned elem, k;
 	GOPath *path;
 	GogTheme *theme = gog_object_get_theme (GOG_OBJECT (model));
-	GOStyle *style;
+	GOStyle *style, *white_style = NULL, *elt_style = NULL;
 	GSList *ptr;
 	unsigned num_series = 0;
 	unsigned index;
@@ -575,6 +726,8 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	double separation_max, separation;
 	GogPieSeriesElement *gpse;
 	GList const *overrides;
+	GogShowNegsMode mode = model->show_negatives;
+	gboolean negative;
 
 	/* compute number of valid series */
 	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
@@ -586,11 +739,16 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	if (num_series <= 0)
 		return;
-
+	
 	separation_max = .0;
 	outline_width_max = .0;
 	if ((style = go_styled_object_get_style (GO_STYLED_OBJECT (series))))
 		outline_width_max = gog_renderer_line_size (view->renderer, style->line.width);
+	if (mode == GOG_SHOW_NEGS_WHITE) {
+		white_style = go_style_dup (style);
+		white_style->fill.type = GO_STYLE_FILL_PATTERN;
+		go_pattern_set_solid (&white_style->fill.pattern, GO_COLOR_WHITE);
+	}
 	for (overrides = gog_series_get_overrides (GOG_SERIES (series));
 	     overrides != NULL;
 	     overrides = overrides->next) {
@@ -690,7 +848,7 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 		/* centre things */
 		cx = view->allocation.x + view->allocation.w/2.;
 		cy = view->allocation.y + view->allocation.h/2.;
-	
+
 		r_tot = view->allocation.h;
 		if (r_tot > view->allocation.w)
 			r_tot = view->allocation.w;
@@ -712,10 +870,10 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			continue;
 		if (index > num_series) /* people snuck extra series into a pie */
 			break;
-		
-		if (num_series == index) 
+
+		if (num_series == index)
 			r -= outline_width_max / 2.0;
-		
+
 		has_hole = center_radius > 0. || index > 1;
 		r_int = center_radius + r * ((double)index - 1.0) / (double)num_series;
 		r_ext = center_radius + r * (double)index / (double)num_series;
@@ -726,13 +884,23 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 		vals = go_data_get_values (series->base.values[1].data);
 
 		style = GOG_STYLED_OBJECT (series)->style;
-		if (model->base.vary_style_by_element)
+		if (model->base.vary_style_by_element || mode == GOG_SHOW_NEGS_WHITE)
 			style = go_style_dup (style);
 		gog_renderer_push_style (view->renderer, style);
 
 		overrides = gog_series_get_overrides (GOG_SERIES (series));
 		for (k = 0 ; k < series->base.num_elements; k++) {
-			len = fabs (vals[k]) * scale;
+			len = vals[k] * scale;
+			negative = len < 0;
+			if (negative) {
+				if (mode == GOG_SHOW_NEGS_SKIP) {
+					if ((overrides != NULL) &&
+					    (GOG_SERIES_ELEMENT (overrides->data)->index == k))
+						overrides = overrides->next;
+					continue;
+				}
+				len = -len;
+			}
 			if (!go_finite (len) || len < 1e-3) {
 				if ((overrides != NULL) &&
 				    (GOG_SERIES_ELEMENT (overrides->data)->index == k))
@@ -745,12 +913,23 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			    (GOG_SERIES_ELEMENT (overrides->data)->index == k)) {
 				gpse = GOG_PIE_SERIES_ELEMENT (overrides->data);
 				overrides = overrides->next;
-				gog_renderer_push_style (view->renderer,
-					go_styled_object_get_style (
-						GO_STYLED_OBJECT (gpse)));
-			} else if (model->base.vary_style_by_element)
-				gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
-							model->base.index_num + k, GO_STYLE_FILL);
+				if (negative && mode == GOG_SHOW_NEGS_WHITE) {
+					elt_style = go_style_dup (go_styled_object_get_style (
+							GO_STYLED_OBJECT (gpse)));
+					elt_style->fill.type = GO_STYLE_FILL_PATTERN;
+					go_pattern_set_solid (&elt_style->fill.pattern, GO_COLOR_WHITE);
+					gog_renderer_push_style (view->renderer, elt_style);
+				} else
+					gog_renderer_push_style (view->renderer,
+						go_styled_object_get_style (
+							GO_STYLED_OBJECT (gpse)));
+			} else {
+				if (negative && mode == GOG_SHOW_NEGS_WHITE)
+					gog_renderer_push_style (view->renderer, white_style);
+				else if (model->base.vary_style_by_element)
+					gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
+								model->base.index_num + k, GO_STYLE_FILL);
+			}
 
 			/* only separate the outer ring */
 			separated_cx = cx;
@@ -776,13 +955,20 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			gog_renderer_draw_shape (view->renderer, path);
 			go_path_clear (path);
 
-			if (gpse != NULL)
+			if (gpse != NULL || (negative && mode == GOG_SHOW_NEGS_WHITE)) {
 				gog_renderer_pop_style (view->renderer);
+				if (elt_style) {
+					g_object_unref (elt_style);
+					elt_style = NULL;
+				}
+			}
 		}
 
 		gog_renderer_pop_style (view->renderer);
 		if (model->base.vary_style_by_element)
 			g_object_unref (style);
+		if (white_style)
+			g_object_unref (white_style);
 
 		index ++;
 	}
@@ -793,19 +979,43 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 static void
 gog_pie_view_build_toolkit (GogView *view)
 {
-	(GOG_VIEW_CLASS (pie_view_parent_klass)->build_toolkit) (view);	
+	(GOG_VIEW_CLASS (pie_view_parent_klass)->build_toolkit) (view);
 #ifdef GOFFICE_WITH_GTK
 	view->toolkit = g_slist_prepend (view->toolkit, &gog_tool_move_pie);
 #endif
 }
 
+static char*
+gog_pie_view_get_tip_at_point (GogView *view, double x, double y)
+{
+	GogPieSeries *series = NULL;
+	int index = gog_pie_view_get_data_at_point (GOG_PLOT_VIEW (view), x, y, (GogSeries** )&series);
+	char *label, *ret;
+	double *vals, value;
+
+	if (index < 0)
+		return NULL;
+	vals = go_data_get_values (series->base.values[1].data);
+	value = fabs (vals[index]);
+	label = series->base.values[0].data? go_data_get_vector_string (series->base.values[0].data, index): NULL;
+	if (label && *label)
+		ret = g_strdup_printf (_("%s: %g (%.2f%%)"), label, value, value * 100 / series->total);
+	else
+		ret = g_strdup_printf (_("%g (%.2f%%)"), value, value * 100 / series->total);
+	g_free (label);
+	return ret;
+}
+
 static void
 gog_pie_view_class_init (GogViewClass *view_klass)
 {
+	GogPlotViewClass *pv_klass = (GogPlotViewClass *) view_klass;
 	pie_view_parent_klass = g_type_class_peek_parent (view_klass);
 
 	view_klass->render 		= gog_pie_view_render;
 	view_klass->build_toolkit 	= gog_pie_view_build_toolkit;
+	view_klass->get_tip_at_point    = gog_pie_view_get_tip_at_point;
+	pv_klass->get_data_at_point     = gog_pie_view_get_data_at_point;
 }
 
 GSF_DYNAMIC_CLASS (GogPieView, gog_pie_view,
@@ -829,10 +1039,10 @@ gog_pie_series_set_property (GObject *obj, guint param_id,
 	GogPieSeries *pie = GOG_PIE_SERIES (obj);
 
 	switch (param_id) {
-	case SERIES_PROP_INITIAL_ANGLE :
+	case SERIES_PROP_INITIAL_ANGLE:
 		pie->initial_angle = g_value_get_float (value);
 		break;
-	case SERIES_PROP_SEPARATION :
+	case SERIES_PROP_SEPARATION:
 		pie->separation = g_value_get_float (value);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -850,10 +1060,10 @@ gog_pie_series_get_property (GObject *obj, guint param_id,
 	GogPieSeries *pie = GOG_PIE_SERIES (obj);
 
 	switch (param_id) {
-	case SERIES_PROP_INITIAL_ANGLE :
+	case SERIES_PROP_INITIAL_ANGLE:
 		g_value_set_float (value, pie->initial_angle);
 		break;
-	case SERIES_PROP_SEPARATION :
+	case SERIES_PROP_SEPARATION:
 		g_value_set_float (value, pie->separation);
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
@@ -868,6 +1078,7 @@ gog_pie_series_update (GogObject *obj)
 	int len = 0;
 	GogPieSeries *series = GOG_PIE_SERIES (obj);
 	unsigned old_num = series->base.num_elements;
+	GogShowNegsMode mode = GOG_PIE_PLOT (series->base.plot)->show_negatives;
 
 	if (series->base.values[1].data != NULL) {
 		vals = go_data_get_values (series->base.values[1].data);
@@ -875,9 +1086,14 @@ gog_pie_series_update (GogObject *obj)
 	}
 	series->base.num_elements = len;
 
-	for (total = 0. ; len-- > 0 ;)
-		if (go_finite (vals[len]))
-			total += fabs (vals[len]);
+	for (total = 0. ; len-- > 0 ;) {
+		double val = vals[len];
+		if (go_finite (val)) {
+			if (val < 0)
+				val = (mode == GOG_SHOW_NEGS_SKIP)? 0.: -val;
+			total += val;
+		}
+	}
 	series->total = total;
 
 	/* queue plot for redraw */
@@ -903,13 +1119,13 @@ gog_pie_series_class_init (GObjectClass *gobject_klass)
 	gobject_klass->get_property = gog_pie_series_get_property;
 
 	g_object_class_install_property (gobject_klass, SERIES_PROP_INITIAL_ANGLE,
-		g_param_spec_float ("initial-angle", 
+		g_param_spec_float ("initial-angle",
 			_("Initial-angle"),
 			_("Degrees clockwise from 12 O'Clock"),
 			0, G_MAXFLOAT, 0.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_SEPARATION,
-		g_param_spec_float ("separation", 
+		g_param_spec_float ("separation",
 			_("Separation"),
 			_("Default amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
